package org.eclipse.osgi.util;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Field;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;

public abstract class NLS {
  private static final Object[] EMPTY_ARGS = new Object[0];
  
  private static final String EXTENSION = ".properties";
  
  private static String[] nlSuffixes;
  
  static final int SEVERITY_ERROR = 4;
  
  static final int SEVERITY_WARNING = 2;
  
  static final Object ASSIGNED = new Object();
  
  public static String bind(String message, Object binding) {
    return internalBind(message, null, String.valueOf(binding), null);
  }
  
  public static String bind(String message, Object binding1, Object binding2) {
    return internalBind(message, null, String.valueOf(binding1), String.valueOf(binding2));
  }
  
  public static String bind(String message, Object[] bindings) {
    return internalBind(message, bindings, null, null);
  }
  
  public static void initializeMessages(String bundleName, Class clazz) {
    if (System.getSecurityManager() == null) {
      load(bundleName, clazz);
      return;
    } 
    AccessController.doPrivileged((PrivilegedAction<?>)new Object(bundleName, clazz));
  }
  
  private static String internalBind(String message, Object[] args, String argZero, String argOne) {
    if (message == null)
      return "No message available."; 
    if (args == null || args.length == 0)
      args = EMPTY_ARGS; 
    int length = message.length();
    int bufLen = length + args.length * 5;
    if (argZero != null)
      bufLen += argZero.length() - 3; 
    if (argOne != null)
      bufLen += argOne.length() - 3; 
    StringBuffer buffer = new StringBuffer((bufLen < 0) ? 0 : bufLen);
    for (int i = 0; i < length; i++) {
      int index, number, nextIndex;
      char next, c = message.charAt(i);
      switch (c) {
        case '{':
          index = message.indexOf('}', i);
          if (index == -1) {
            buffer.append(c);
            break;
          } 
          i++;
          if (i >= length) {
            buffer.append(c);
            break;
          } 
          number = -1;
          try {
            number = Integer.parseInt(message.substring(i, index));
          } catch (NumberFormatException numberFormatException) {
            throw new IllegalArgumentException();
          } 
          if (number == 0 && argZero != null) {
            buffer.append(argZero);
          } else if (number == 1 && argOne != null) {
            buffer.append(argOne);
          } else {
            if (number >= args.length || number < 0) {
              buffer.append("<missing argument>");
              i = index;
              break;
            } 
            buffer.append(args[number]);
          } 
          i = index;
          break;
        case '\'':
          nextIndex = i + 1;
          if (nextIndex >= length) {
            buffer.append(c);
            break;
          } 
          next = message.charAt(nextIndex);
          if (next == '\'') {
            i++;
            buffer.append(c);
            break;
          } 
          index = message.indexOf('\'', nextIndex);
          if (index == -1) {
            buffer.append(c);
            break;
          } 
          buffer.append(message.substring(nextIndex, index));
          i = index;
          break;
        default:
          buffer.append(c);
          break;
      } 
    } 
    return buffer.toString();
  }
  
  private static String[] buildVariants(String root) {
    if (nlSuffixes == null) {
      String nl = Locale.getDefault().toString();
      ArrayList<String> result = new ArrayList(4);
      while (true) {
        result.add(String.valueOf('_') + nl + ".properties");
        int lastSeparator = nl.lastIndexOf('_');
        if (lastSeparator == -1)
          break; 
        nl = nl.substring(0, lastSeparator);
      } 
      result.add(".properties");
      nlSuffixes = result.<String>toArray(new String[result.size()]);
    } 
    root = root.replace('.', '/');
    String[] variants = new String[nlSuffixes.length];
    for (int i = 0; i < variants.length; i++)
      variants[i] = String.valueOf(root) + nlSuffixes[i]; 
    return variants;
  }
  
  private static void computeMissingMessages(String bundleName, Class clazz, Map fieldMap, Field[] fieldArray, boolean isAccessible) {
    int numFields = fieldArray.length;
    for (int i = 0; i < numFields; i++) {
      Field field = fieldArray[i];
      if ((field.getModifiers() & 0x19) == 9)
        if (fieldMap.get(field.getName()) != ASSIGNED)
          try {
            String value = "NLS missing message: " + field.getName() + " in: " + bundleName;
            log(2, value, null);
            if (!isAccessible)
              field.setAccessible(true); 
            field.set(null, value);
          } catch (Exception e) {
            log(4, "Error setting the missing message value for: " + field.getName(), e);
          }   
    } 
  }
  
  static void load(String bundleName, Class clazz) {
    System.currentTimeMillis();
    Field[] fieldArray = clazz.getDeclaredFields();
    ClassLoader loader = clazz.getClassLoader();
    boolean isAccessible = ((clazz.getModifiers() & 0x1) != 0);
    int len = fieldArray.length;
    Map<Object, Object> fields = new HashMap<Object, Object>(len * 2);
    for (int i = 0; i < len; i++)
      fields.put(fieldArray[i].getName(), fieldArray[i]); 
    String[] variants = buildVariants(bundleName);
    for (int j = 0; j < variants.length; j++) {
      InputStream input = (loader == null) ? ClassLoader.getSystemResourceAsStream(variants[j]) : loader.getResourceAsStream(variants[j]);
      if (input != null)
        try {
          MessagesProperties properties = new MessagesProperties(fields, bundleName, isAccessible);
          properties.load(input);
        } catch (IOException e) {
          log(4, "Error loading " + variants[j], e);
        } finally {
          if (input != null)
            try {
              input.close();
            } catch (IOException iOException) {} 
        }  
    } 
    computeMissingMessages(bundleName, clazz, fields, fieldArray, isAccessible);
  }
  
  static void log(int severity, String message, Exception e) {
    String statusMsg;
    switch (severity) {
      case 4:
        statusMsg = "Error: ";
        break;
      default:
        statusMsg = "Warning: ";
        break;
    } 
    if (message != null)
      statusMsg = String.valueOf(statusMsg) + message; 
    if (e != null)
      statusMsg = String.valueOf(statusMsg) + ": " + e.getMessage(); 
    System.err.println(statusMsg);
    if (e != null)
      e.printStackTrace(); 
  }
}
